//+------------------------------------------------------------------+
//|                                                      Clogger.mqh |
//|                                                             ProF |
//|                                                          http:// |
//+------------------------------------------------------------------+
#property copyright "ProF"
#property link      "http://"

//   ()
#define MAX_CACHE_SIZE	 10000
//    
#define MAX_FILE_SIZEMB	10
//+------------------------------------------------------------------+
//|                                                            |
//+------------------------------------------------------------------+
class CLogger
  {
private:
   string            project,file; //    
   string            logCache[MAX_CACHE_SIZE]; //  
   int               sizeCache;  // 
   int               cacheTimeLimit;  // 
   datetime          cacheTime;   //     
   int               handleFile; //  
   string            defCategory;//  
   void              writeLog(string log_msg); //        
public:
   void              CLogger(void){cacheTimeLimit=0; cacheTime=0; sizeCache=0;};//
   void             ~CLogger(void){};//
   void              SetSetting(string project,string file_name,
                                string default_category="",int cache_time_limit=0);// 
   void              init(); //,    
   void              deinit(); //,  
   void              write(string msg,string category=""); // 
   void              write(string msg,string category,color colorOfMsg,string file="",int line=0); // 
   void              write(string msg,string category,uchar red,uchar green,uchar blue,
                           string file="",int line=0); // 
   void              flush(void); //   

  };
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
void CLogger::SetSetting(string project_name,string file_name,
                        string default_category="",int cache_time_limit=0)
  {
   project=project_name; // 
   file=file_name; // 
   cacheTimeLimit=cache_time_limit;  // 
   if(default_category=="") //   
     {  defCategory="notice";   }
     else
     {defCategory = default_category;}
  }
//+------------------------------------------------------------------+
//|                                                     |
//+------------------------------------------------------------------+
void CLogger::init(void)
  {
   string path;
   MqlDateTime date;
   int i=0;
   TimeToStruct(TimeCurrent(),date); //  
   StringConcatenate(path,"log\\log_",project,"\\log_",file,"_",
                     date.year,date.mon,date.day); //    
   handleFile=FileOpen(path+".txt",FILE_WRITE|FILE_READ|
                       FILE_UNICODE|FILE_TXT|FILE_SHARE_READ); //   
   while(FileSize(handleFile)>(MAX_FILE_SIZEMB*1000000)) //  
     {
      //     
      i++;
      FileClose(handleFile);
      handleFile=FileOpen(path+"_"+(string)i+".txt",
                          FILE_WRITE|FILE_READ|FILE_UNICODE|FILE_TXT|FILE_SHARE_READ);
     }
   FileSeek(handleFile,0,SEEK_END); //    
  }
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
void CLogger::deinit(void)
  {
   FileClose(handleFile); // 
  }
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+
void CLogger::writeLog(string log_msg)
  {
   if(cacheTimeLimit!=0) //   
     {
      if((sizeCache<MAX_CACHE_SIZE-1 && TimeCurrent()-cacheTime<cacheTimeLimit)
         || sizeCache==0) //          
        {
         //   
         logCache[sizeCache++]=log_msg;
        }
      else
        {
         //        
         logCache[sizeCache++]=log_msg;
         flush();
        }

     }
   else
     {
      // ,    
      FileWrite(handleFile,log_msg);
     }
   if(FileTell(handleFile)>(MAX_FILE_SIZEMB*1000000)) //   
     {
      //    ,     
      deinit();
      init();
     }
  }
//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
void CLogger::write(string msg,string category="")
  {
   string msg_log;
   if(category=="") //   
     {   category=defCategory;   } //   

//       
   StringConcatenate(msg_log,category,":|:",TimeToString(TimeCurrent(),TIME_SECONDS),"    ",msg);
   writeLog(msg_log);
  }
//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
void CLogger::write(string msg,string category,color colorOfMsg,string file="",int line=0)
  {
   string msg_log;
   int red,green,blue;
   red=(colorOfMsg  &Red); //    
   green=(colorOfMsg  &0x00FF00)>>8; //    
   blue=(colorOfMsg  &Blue)>>16; //    
                                 //     ,        
   if(file!="" && line!=0)
     {
      StringConcatenate(msg_log,category,":|:",red,",",green,",",blue,
                        ":|:",TimeToString(TimeCurrent(),TIME_SECONDS),"    ",
                        ": ",file,"   : ",line,"   ",msg);
     }
   else
     {
      StringConcatenate(msg_log,category,":|:",red,",",green,",",blue,
                        ":|:",TimeToString(TimeCurrent(),TIME_SECONDS),"    ",msg);
     }
   writeLog(msg_log);
  }
//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
void CLogger::write(string msg,string category,uchar red,uchar green,uchar blue,string file="",int line=0)
  {
   string msg_log;

//     ,        
   if(file!="" && line!=0)
     {
      StringConcatenate(msg_log,category,":|:",red,",",green,",",blue,
                        ":|:",TimeToString(TimeCurrent(),TIME_SECONDS),"    ",
                        ": ",file,"   : ",line,"   ",msg);
     }
   else
     {
      StringConcatenate(msg_log,category,":|:",red,",",green,",",blue,
                        ":|:",TimeToString(TimeCurrent(),TIME_SECONDS),"    ",msg);
     }
   writeLog(msg_log);
  }
//+------------------------------------------------------------------+
//|                                                    |
//+------------------------------------------------------------------+
void CLogger::flush(void)
  {
   for(int i=0;i<sizeCache;i++) //      
     {
      FileWrite(handleFile,logCache[i]);
     }
   sizeCache=0; //  
   cacheTime=TimeCurrent(); //   
  }
//+------------------------------------------------------------------+
